/*
 * mql.re
 *
 * MQL Lexer based on re2c
 *
 * Ulrik Petersen
 * Created: 6/23-2007
 * Last update: 10/8-2007
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2007  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, license version 2.  
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *   02111-1307 USA
 *
 *
 *   Special exception
 *   =================
 * 
 *   In addition, as a special exception, Ulrik Petersen, the
 *   copyright holder of Emdros, gives permission to link Emdros, in
 *   whole or in part, with the libraries which are normally
 *   distributed with:
 *   
 *   a) Sun's Java platform,
 *   b) Python, 
 *   c) Jython,
 *   d) Ruby, and/or 
 *   e) Perl 
 *   f) PostgreSQL
 *   g) OpenSSL
 *
 *   (or with modified versions of these), and to distribute linked
 *   combinations including both Emdros, in whole or in part, and one
 *   or more of the libraries normally distributed with (a)-(g) above.
 *
 *   Please note: This gives you special rights concerning the
 *   libraries which normally accompany the above pieces of software.
 *   It gives you no special rights concerning software that you write
 *   yourself.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than the libraries
 *   normally distributed with (a)-(g) above.
 *
 *   If you modify this file, you may extend this exception to your
 *   version of the file, but you are not obligated to do so. If you
 *   do not wish to do so, delete this exception statement from your
 *   version.
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 *
 *   Website
 *   =======
 *
 *   Emdros has a website here:
 *
 *   http://emdros.org
 *
 *
 *
 **************************************************************************/

/**@file mql_lexer.re
 *@brief Implementation file for MQL Lexer (MQL layer)
 */

#include <cstdlib>
#include <cstring>
#include <mql_lexer.h>
#include <mqlpars.h>
#include <iostream>
#include <string_func.h>

#define YYCTYPE         unsigned char
#define YYCURSOR        cur
#define YYLIMIT         lim
#define YYMARKER        ptr
/*!re2c
re2c:yyfill:enable = 0;
re2c:labelprefix = xx;
*/


/*!re2c
zero     = "\000";
any      = [\001-\377];
dot      = any\[\n];
BACKPING = [\x60]; /* Backping "`" */
SLASH    = [\\];   /* slash "\\" */
A        = [Aa];
B        = [Bb];
C        = [Cc];
D        = [Dd];
E        = [Ee];
F        = [Ff];
G        = [Gg];
H        = [Hh];
I        = [Ii];
J        = [Jj];
K        = [Kk];
L        = [Ll];
M        = [Mm];
N        = [Nn];
O        = [Oo];
P        = [Pp];
Q        = [Qq];
R        = [Rr];
S        = [Ss];
T        = [Tt];
U        = [Uu];
V        = [Vv];
W        = [Ww];
X        = [Xx];
Y        = [Yy];
Z        = [Zz];
*/


MQLScanner::MQLScanner(const std::string& strIn)
{
	sz = strIn.c_str();
	lim = sz + strIn.length();
	bot = tok = ptr = pos = top = eof = 0;
	cur = sz;
}

MQLScanner::~MQLScanner()
{
	if (bot) {
		delete[] bot;
	}
}

/*
char *MQLScanner::fill(char *cursor)
{
        if(!eof)
        {
                unsigned int cnt = tok - bot;
                if(cnt)
                {
                        memcpy(bot, tok, lim - tok);
                        tok = bot;
                        ptr -= cnt;
                        cursor -= cnt;
                        pos -= cnt;
                        lim -= cnt;
                }
                if((top - lim) < BSIZE)
                {
                        char *buf = new char[(lim - bot) + BSIZE];
                        memcpy(buf, tok, lim - tok);
                        tok = buf;
                        ptr = &buf[ptr - bot];
                        cursor = &buf[cursor - bot];
                        pos = &buf[pos - bot];
                        lim = &buf[lim - bot];
                        top = &lim[BSIZE];
                        delete [] bot;
                        bot = buf;
                }
                pIn->read(lim, BSIZE);
                if ((cnt = pIn->gcount()) != BSIZE )
                {
                        eof = &lim[cnt]; *eof++ = '\0';
                }
                lim += cnt;
        }
        return cursor;
}
*/

#define TOKEN_RETURN(X,Y) { yylval->setName(X); return Y; }

int MQLScanner::scan(Token *pYYLVAL)
{
	yylval = pYYLVAL;

 scan:	
	tok = cur;

/*!re2c
C R E A T E                 { TOKEN_RETURN("create", T_KEY_CREATE); }
U P D A T E                 { TOKEN_RETURN("update", T_KEY_UPDATE); }
S E L E C T                 { TOKEN_RETURN("select", T_KEY_SELECT);  }
D R O P                     { TOKEN_RETURN("drop", T_KEY_DROP);  }
G E T                       { TOKEN_RETURN("get", T_KEY_GET);  }
I N S E R T                 { TOKEN_RETURN("insert", T_KEY_INSERT);  }
D E L E T E                 { TOKEN_RETURN("delete", T_KEY_DELETE);  }
D A T A B A S E             { TOKEN_RETURN("database", T_KEY_DATABASE);  }
U S E                       { TOKEN_RETURN("use", T_KEY_USE);  }
I N I T I A L I Z E         { TOKEN_RETURN("initialize", T_KEY_INITIALIZE);  }
V A C U U M                 { TOKEN_RETURN("vacuum", T_KEY_VACUUM); }
A N A L Y Z E               { TOKEN_RETURN("analyze", T_KEY_ANALYZE); }
B E G I N                   { TOKEN_RETURN("begin", T_KEY_BEGIN); }
C O M M I T                 { TOKEN_RETURN("commit", T_KEY_COMMIT); }
A B O R T                   { TOKEN_RETURN("abort", T_KEY_ABORT); }
T R A N S A C T I O N       { TOKEN_RETURN("transaction", T_KEY_TRANSACTION); }
T Y P E                     { TOKEN_RETURN("type", T_KEY_TYPE);  }
T Y P E S                   { TOKEN_RETURN("types", T_KEY_TYPES);  }
O B J E C T                 { TOKEN_RETURN("object", T_KEY_OBJECT);  }
O B J E C T S               { TOKEN_RETURN("objects", T_KEY_OBJECTS);  }
L I S T                     { TOKEN_RETURN("list", T_KEY_LIST); }
O F                         { TOKEN_RETURN("of", T_KEY_OF); }
I N                         { TOKEN_RETURN("in", T_KEY_IN);  }
I N D E X E S               { TOKEN_RETURN("indexes", T_KEY_INDEXES); }
I N D E X                   { TOKEN_RETURN("index", T_KEY_INDEX); }
I N T E G E R               { TOKEN_RETURN("integer", T_KEY_INTEGER);  }
S T R I N G                 { TOKEN_RETURN("string", T_KEY_STRING);  }
A S C I I                   { TOKEN_RETURN("ascii", T_KEY_ASCII); }
I D "_" D                   { TOKEN_RETURN("id_d", T_KEY_ID_D);  }
I D "_" D S                 { TOKEN_RETURN("id_ds", T_KEY_ID_DS);  }
D E F A U L T               { TOKEN_RETURN("default", T_KEY_DEFAULT);  }
C O M P U T E D             { TOKEN_RETURN("computed", T_KEY_COMPUTED);  }
A D D                       { TOKEN_RETURN("add", T_KEY_ADD);  }
R E M O V E                 { TOKEN_RETURN("remove", T_KEY_REMOVE);  }
F R O M                     { TOKEN_RETURN("from", T_KEY_FROM);  }
B Y                         { TOKEN_RETURN("by", T_KEY_BY);  }
E N U M                     { TOKEN_RETURN("enum", T_KEY_ENUM);  }
E N U M E R A T I O N       { TOKEN_RETURN("enumeration", T_KEY_ENUMERATION);  }
E N U M E R A T I O N S     { TOKEN_RETURN("enumerations", T_KEY_ENUMERATIONS);  }
C O N S T A N T S           { TOKEN_RETURN("constants", T_KEY_CONSTANTS);  }
S E G M E N T               { TOKEN_RETURN("segment", T_KEY_SEGMENT);  }
R A N G E                   { TOKEN_RETURN("range", T_KEY_RANGE);  }
S I N G L E                 { TOKEN_RETURN("single", T_KEY_SINGLE); }
M U L T I P L E             { TOKEN_RETURN("multiple", T_KEY_MULTIPLE); }
F O C U S                   { TOKEN_RETURN("focus", T_KEY_FOCUS);  }
A L L                       { TOKEN_RETURN("all", T_KEY_ALL);  }
H A S                       { TOKEN_RETURN("has", T_KEY_HAS);  }
W H E R E                   { TOKEN_RETURN("where", T_KEY_WHERE);  }
H A V I N G                 { TOKEN_RETURN("having", T_KEY_HAVING); }
F E A T U R E               { TOKEN_RETURN("feature", T_KEY_FEATURE);  }
F E A T U R E S             { TOKEN_RETURN("features", T_KEY_FEATURES);  }
M O N A D S                 { TOKEN_RETURN("monads", T_KEY_MONADS);  }
M O N A D                   { TOKEN_RETURN("monad", T_KEY_MONAD);  }
U N I Q U E                 { TOKEN_RETURN("unique", T_KEY_UNIQUE);  }
U S I N G                   { TOKEN_RETURN("using", T_KEY_USING);  }
W I T H                     { TOKEN_RETURN("with", T_KEY_WITH);  }
W I T H O U T               { TOKEN_RETURN("without", T_KEY_WITHOUT);  }
K E Y                       { TOKEN_RETURN("key", T_KEY_KEY); }
N O R E T R I E V E         { TOKEN_RETURN("noretrieve", T_KEY_NORETRIEVE);  }
R E T R I E V E             { TOKEN_RETURN("retrieve", T_KEY_RETRIEVE);  }
F I R S T                   { TOKEN_RETURN("first", T_KEY_FIRST);  }
L A S T                     { TOKEN_RETURN("last", T_KEY_LAST);  }
O R                         { TOKEN_RETURN("or", T_KEY_OR);  }
A N D                       { TOKEN_RETURN("and", T_KEY_AND);  }
N O T                       { TOKEN_RETURN("not", T_KEY_NOT);  }
N O T E X I S T             { TOKEN_RETURN("notexist", T_KEY_NOTEXIST);  }
N O T E X I S T S           { TOKEN_RETURN("notexists", T_KEY_NOTEXISTS);  }
G A P                       { TOKEN_RETURN("gap", T_KEY_GAP);  }
G A P "?"                   { TOKEN_RETURN("gap?", T_KEY_OPT_GAP);  }
"\.\."                      { TOKEN_RETURN("..", T_KEY_POWER);  }
B E T W E E N               { TOKEN_RETURN("between", T_KEY_BETWEEN); }
A S                         { TOKEN_RETURN("as", T_KEY_AS);  }
N I L                       { TOKEN_RETURN("nil", T_KEY_NIL);  }
G O                         { TOKEN_RETURN("go", T_KEY_GO); }
R E T U R N I N G           { TOKEN_RETURN("returning", T_KEY_RETURNING); }
C A L C U L A T I O N       { TOKEN_RETURN("calculation", T_KEY_CALCULATION); }
M I N "_" M                 { TOKEN_RETURN("min_m", T_KEY_MIN_M); }
M A X "_" M                 { TOKEN_RETURN("max_m", T_KEY_MAX_M); }
F L A T                     { TOKEN_RETURN("flat", T_KEY_FLAT); }
F U L L                     { TOKEN_RETURN("full", T_KEY_FULL); }
O N                         { TOKEN_RETURN("on", T_KEY_ON); }
S H E A F                   { TOKEN_RETURN("sheaf", T_KEY_SHEAF); }
A T                         { TOKEN_RETURN("at", T_KEY_AT); }
Q U I T                     { TOKEN_RETURN("quit", T_KEY_QUIT); }
S E T                       { TOKEN_RETURN("set", T_KEY_SET); }
S E T S                     { TOKEN_RETURN("sets", T_KEY_SETS); }
U N I O N                   { TOKEN_RETURN("union", T_KEY_UNION); }
I N T E R S E C T           { TOKEN_RETURN("intersect", T_KEY_INTERSECT); }
D I F F E R E N C E         { TOKEN_RETURN("difference", T_KEY_DIFFERENCE); }
R E P L A C E               { TOKEN_RETURN("replace", T_KEY_REPLACE); }
E N C O D I N G             { TOKEN_RETURN("encoding", T_KEY_ENCODING); }
"="                         { TOKEN_RETURN("=", T_KEY_EQUALS);  }
"<"                         { TOKEN_RETURN("<", T_KEY_LESS_THAN);  }
">"                         { TOKEN_RETURN(">", T_KEY_GREATER_THAN);  }
"<>"                        { TOKEN_RETURN("<>", T_KEY_NOT_EQUAL);  }
"<="                        { TOKEN_RETURN("<=", T_KEY_LESS_THAN_OR_EQUAL);  }
"=<"                        { TOKEN_RETURN("=<", T_KEY_LESS_THAN_OR_EQUAL);  }
">="                        { TOKEN_RETURN(">=", T_KEY_GREATER_THAN_OR_EQUAL);  }
"=>"                        { TOKEN_RETURN("=>", T_KEY_GREATER_THAN_OR_EQUAL);  }
"\~"                        { TOKEN_RETURN("~", T_KEY_TILDE); }
"!\~"                       { TOKEN_RETURN("!~", T_KEY_NOT_TILDE); }
"\["                        { TOKEN_RETURN("[", T_KEY_OPEN_SQUARE_BRACKET); }
"\]"                        { TOKEN_RETURN("]", T_KEY_CLOSE_SQUARE_BRACKET); }
"{"                         { TOKEN_RETURN("{", T_KEY_OPEN_BRACE);  }
"}"                         { TOKEN_RETURN("}", T_KEY_CLOSE_BRACE);  }
"("                         { TOKEN_RETURN("(", T_KEY_OPEN_BRACKET); }
")"                         { TOKEN_RETURN(")", T_KEY_CLOSE_BRACKET); }
":="                        { TOKEN_RETURN(":=", T_KEY_ASSIGN);  }
":"                         { TOKEN_RETURN(":", T_KEY_COLON); }
";"                         { TOKEN_RETURN(";", T_KEY_SEMICOLON); }
","                         { TOKEN_RETURN(",", T_KEY_COMMA); }
"."                         { TOKEN_RETURN(".", T_KEY_DOT); }
"\*"                        { TOKEN_RETURN("*", T_KEY_STAR); }
"-"                         { TOKEN_RETURN("-", T_KEY_DASH); }
"!"                         { TOKEN_RETURN("!", T_KEY_EXCLAMATION); }
[a-zA-Z_][a-zA-Z0-9_]*      { addToken();
			      TOKEN_RETURN("identifier", T_IDENTIFIER);
			    }
(BACKPING [a-zA-Z_][a-zA-Z0-9_]*)+       { addToken();
                                           TOKEN_RETURN("mark", T_MARK); }
[0-9]{1,10}                 { addInteger();
                              TOKEN_RETURN(INTEGER_MAGIC, T_INTEGER); }
'"'                         { yylval->setString(new std::string);
			      goto stringdq; 
                            }
[']                         { yylval->setString(new std::string);
			      goto stringsq; 
			    }
" "                         { goto scan; }
"\t"                        { goto scan; }
"\r"                        { goto scan; }
"\n"                        { goto scan; }
zero      { if (cur >= lim) { goto end; } }
dot       { yylval->setChar(*tok);
	    TOKEN_RETURN(CHAR_MAGIC, *tok); }
 */

stringdq:
	tok = cur;
/*!re2c
'"'                   { TOKEN_RETURN("string", T_STRING); }
SLASH "n"             { yylval->pString->append(1, '\n'); goto stringdq; }
SLASH "t"             { yylval->pString->append(1, '\t'); goto stringdq; }
SLASH "v"             { yylval->pString->append(1, '\v'); goto stringdq; }
SLASH "b"             { yylval->pString->append(1, '\b'); goto stringdq; }
SLASH "a"             { yylval->pString->append(1, '\a'); goto stringdq; }
SLASH "r"             { yylval->pString->append(1, '\r'); goto stringdq; }
SLASH "f"             { yylval->pString->append(1, '\f'); goto stringdq; }
SLASH SLASH           { yylval->pString->append(1, '\\'); goto stringdq; }
SLASH "?"             { yylval->pString->append(1, '\?'); goto stringdq; }
SLASH "'"             { yylval->pString->append(1, '\''); goto stringdq; }
SLASH '"'             { yylval->pString->append(1, '\"'); goto stringdq; }
SLASH [0-3][0-7][0-7] { tok += 1; // Go past SLASH 
		    std::string mystr;
		    getString(mystr);
		    yylval->pString->append(1, octal2char(mystr.c_str())); 
		    goto stringdq; }
SLASH "x" [A-Fa-f0-9][A-Fa-f0-9]    { tok += 2; // Go past SLASH x
			       std::string mystr;
			       getString(mystr);
			       yylval->pString->append(1, hex2char(mystr.c_str()));
			       goto stringdq; 			     
                             }
zero                         { if (cur >= lim) { goto end; } }
dot                          { yylval->pString->append(1, *tok);
			       goto stringdq; }
"\n"                         { yylval->pString->append(1, '\n');
			       goto stringdq; }
*/

stringsq:
	tok = cur;
/*!re2c
[']                          { TOKEN_RETURN("string", T_STRING); }
zero                         { if (cur >= lim) { goto end; } }
"\n"                         { yylval->pString->append(1, '\n');
			       goto stringsq; }
dot\[']                      { yylval->pString->append(1, *tok);
			       goto stringsq; }
*/
	
 end:
	return 0;
}


void MQLScanner::addToken(void)
{
	yylval->setString(new std::string);
	const char *p;
	for (p = tok;
	     p != cur;
	     ++p) {
	     yylval->pString->append(1, *p);
	}
}

void MQLScanner::addInteger(void)
{
	const unsigned int MAX_INTEGER_CHARS = 12;
	unsigned int cnt = cur - tok;
	if (cnt > MAX_INTEGER_CHARS) {
	   	std::string mystring;
		const char *p;
		for (p = tok;
		     p != cur;
	     	     ++p) {
		     mystring.append(1, *p);
		}
		yylval->setInteger(strtol(mystring.c_str(), (char **)NULL, 10));
        } else {
		char szInt[MAX_INTEGER_CHARS+1];

		const char *p;
		char *q = szInt;
		for (p = tok;
		     p != cur;
	     	     ++p) {
		     *q++ = *p;
		}
		*q = '\0';
		yylval->setInteger(strtol(szInt, (char **)NULL, 10));
	}
}

void MQLScanner::getString(std::string& str)
{
	const char *p;
	for (p = tok;
	     p != cur;
	     ++p) {
	     str.append(1, *p);
	}
}

