/*
 * templatelang.re
 *
 * Template language lexer based on re2c
 *
 * Ulrik Sandborg-Petersen
 * Created: 7/28-2008
 * Last update: 12/8-2008
 *
 */
#include <templatelang_lexer.h>
#include "templatelangpars.h"
#include <iostream>
#include <string_func.h>

#define YYCTYPE         unsigned char
#define YYCURSOR        cur
#define YYLIMIT         lim
#define YYMARKER        ptr
/*!re2c
re2c:yyfill:enable = 0;
re2c:labelprefix = tt;
*/


/*!re2c
zero     = "\000";
any      = [\001-\377];
dot      = any\[\n];
BACKPING = [\x60]; /* Backping "`" */
SLASH    = [\\];   /* slash "\\" */
A        = [Aa];
B        = [Bb];
C        = [Cc];
D        = [Dd];
E        = [Ee];
F        = [Ff];
G        = [Gg];
H        = [Hh];
I        = [Ii];
J        = [Jj];
K        = [Kk];
L        = [Ll];
M        = [Mm];
N        = [Nn];
O        = [Oo];
P        = [Pp];
Q        = [Qq];
R        = [Rr];
S        = [Ss];
T        = [Tt];
U        = [Uu];
V        = [Vv];
W        = [Ww];
X        = [Xx];
Y        = [Yy];
Z        = [Zz];
*/


TemplateLanguageScanner::TemplateLanguageScanner(const std::string& strIn)
{
	sz = strIn.c_str();
	lim = sz + strIn.length();
	bot = tok = ptr = pos = top = eof = 0;
	cur = sz;
}

TemplateLanguageScanner::~TemplateLanguageScanner()
{
	if (bot) {
		delete[] bot;
	}
}

/*
char *TemplateLanguageScanner::fill(char *cursor)
{
        if(!eof)
        {
                unsigned int cnt = tok - bot;
                if(cnt)
                {
                        memcpy(bot, tok, lim - tok);
                        tok = bot;
                        ptr -= cnt;
                        cursor -= cnt;
                        pos -= cnt;
                        lim -= cnt;
                }
                if((top - lim) < BSIZE)
                {
                        char *buf = new char[(lim - bot) + BSIZE];
                        memcpy(buf, tok, lim - tok);
                        tok = buf;
                        ptr = &buf[ptr - bot];
                        cursor = &buf[cursor - bot];
                        pos = &buf[pos - bot];
                        lim = &buf[lim - bot];
                        top = &lim[BSIZE];
                        delete [] bot;
                        bot = buf;
                }
                pIn->read(lim, BSIZE);
                if ((cnt = pIn->gcount()) != BSIZE )
                {
                        eof = &lim[cnt]; *eof++ = '\0';
                }
                lim += cnt;
        }
        return cursor;
}
*/

#define TOKEN_RETURN(X,Y) { yylval->setName(X); return Y; }

int TemplateLanguageScanner::scan(Token *pYYLVAL)
{
	yylval = pYYLVAL;

 scan:	
	tok = cur;

/*!re2c
" }}"                  { TOKEN_RETURN("}}", T_KEY_BB_CLOSE); }
"{{ else }}"              { TOKEN_RETURN("else", T_KEY_ELSE); }
"{{ endif }}"             { TOKEN_RETURN("endif", T_KEY_ENDIF); }
"{{ id_d }}"              { TOKEN_RETURN("id_d", T_KEY_ID_D); }
"{{ firstmonad }}"        { TOKEN_RETURN("firstmonad", T_KEY_FIRSTMONAD); }
"{{ dbname }}"            { TOKEN_RETURN("dbname", T_KEY_DBNAME); }
"{{ counter"           { TOKEN_RETURN("counter", T_KEY_COUNTER); }
"{{ listappendbegin"        { TOKEN_RETURN("listappendbegin", T_KEY_LISTAPPENDBEGIN); }
"{{ listappendend }}"        { TOKEN_RETURN("listappendend", T_KEY_LISTAPPENDEND); }
"{{ listemit"          { TOKEN_RETURN("listemit", T_KEY_LISTEMIT); }
"{{ inccounter"        { TOKEN_RETURN("inccounter", T_KEY_INCCOUNTER); }
"{{ feature"           { TOKEN_RETURN("feature", T_KEY_FEATURE); }
"{{ featurenomangle"   { TOKEN_RETURN("featurenomangle", T_KEY_FEATURENOMANGLE); }
"{{ attrib"            { TOKEN_RETURN("attrib", T_KEY_ATTRIB); }
"{{ if featureequal"   { TOKEN_RETURN("featureequal", T_KEY_IF_FEATUREEQUAL); }
"{{ if hasattrib"      { TOKEN_RETURN("hasattrib", T_KEY_IF_HASATTRIB); }
"{{ if attribequal"    { TOKEN_RETURN("attribequal", T_KEY_IF_ATTRIBEQUAL); }
[A-Za-z_][A-Za-z0-9_]* { addToken(); TOKEN_RETURN("identifier", T_IDENTIFIER);}
[0-9][0-9]*            { addInteger();
                         TOKEN_RETURN(INTEGER_MAGIC, T_INTEGER); }
"'"                    { yylval->setString(new std::string);
		         goto stringsq; 
                       }
" "                    { TOKEN_RETURN(" ", T_KEY_SPACE); }
"\t"                   { goto scan; }
"\r"                   { goto scan; }
"\n"                   { TOKEN_RETURN("\\n", T_KEY_NEWLINE); }
zero                   { if (cur >= lim) { goto end; } }
dot                    { yylval->setChar(*tok); TOKEN_RETURN(CHAR_MAGIC, T_CHAR);}
 */


stringsq:
	tok = cur;
/*!re2c
"'"                   { TOKEN_RETURN("string", T_STRING); }
SLASH "n"             { yylval->pString->append(1, '\n'); goto stringsq; }
SLASH "t"             { yylval->pString->append(1, '\t'); goto stringsq; }
SLASH "v"             { yylval->pString->append(1, '\v'); goto stringsq; }
SLASH "b"             { yylval->pString->append(1, '\b'); goto stringsq; }
SLASH "a"             { yylval->pString->append(1, '\a'); goto stringsq; }
SLASH "r"             { yylval->pString->append(1, '\r'); goto stringsq; }
SLASH "f"             { yylval->pString->append(1, '\f'); goto stringsq; }
SLASH SLASH           { yylval->pString->append(1, '\\'); goto stringsq; }
SLASH "?"             { yylval->pString->append(1, '\?'); goto stringsq; }
SLASH "'"             { yylval->pString->append(1, '\''); goto stringsq; }
SLASH '"'             { yylval->pString->append(1, '\"'); goto stringsq; }
SLASH [0-3][0-7][0-7] { tok += 1; // Go past SLASH 
		    std::string mystr;
		    getString(mystr);
		    yylval->pString->append(1, octal2char(mystr.c_str())); 
		    goto stringsq; }
SLASH "x" [A-Fa-f0-9][A-Fa-f0-9]    { tok += 2; // Go past SLASH x
			       std::string mystr;
			       getString(mystr);
			       yylval->pString->append(1, hex2char(mystr.c_str()));
			       goto stringsq; 			     
                             }
zero                         { if (cur >= lim) { goto end; } }
dot                          { yylval->pString->append(1, *tok);
			       goto stringsq; }
"\n"                         { yylval->pString->append(1, '\n');
			       goto stringsq; }
*/


 end:
	return 0;
}


void TemplateLanguageScanner::addToken(void)
{
	yylval->setString(new std::string);
	const char *p;
	for (p = tok;
	     p != cur;
	     ++p) {
	     yylval->pString->append(1, *p);
	}
}

void TemplateLanguageScanner::addInteger(void)
{
	const unsigned int MAX_INTEGER_CHARS = 12;
	unsigned int cnt = cur - tok;
	if (cnt > MAX_INTEGER_CHARS) {
	   	std::string mystring;
		const char *p;
		for (p = tok;
		     p != cur;
	     	     ++p) {
		     mystring.append(1, *p);
		}
		yylval->setInteger(strtol(mystring.c_str(), (char **)NULL, 10));
		yylval->setString(new std::string(mystring));
        } else {
		char szInt[MAX_INTEGER_CHARS+1];

		const char *p;
		char *q = szInt;
		for (p = tok;
		     p != cur;
	     	     ++p) {
		     *q++ = *p;
		}
		*q = '\0';
		yylval->setInteger(strtol(szInt, (char **)NULL, 10));
		yylval->setString(new std::string(szInt));
	}
}

void TemplateLanguageScanner::getString(std::string& str)
{
	const char *p;
	for (p = tok;
	     p != cur;
	     ++p) {
	     str.append(1, *p);
	}
}

