/*
 * templatelang_classes.cpp
 *
 * Template language classes
 *
 * Ulrik Sandborg-Petersen
 * Created: 7/28-2008
 * Last update: 3/9-2009
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2008-2009  Ulrik Sandborg-Petersen
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, license version 2.  
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *   02111-1307 USA
 *
 *
 *   Special exception
 *   =================
 * 
 *   In addition, as a special exception, Ulrik Petersen, the
 *   copyright holder of Emdros, gives permission to link Emdros, in
 *   whole or in part, with the libraries which are normally
 *   distributed with:
 *   
 *   a) Sun's Java platform,
 *   b) Python, 
 *   c) Jython,
 *   d) Ruby, and/or 
 *   e) Perl 
 *   f) PostgreSQL
 *   g) OpenSSL
 *
 *   (or with modified versions of these), and to distribute linked
 *   combinations including both Emdros, in whole or in part, and one
 *   or more of the libraries normally distributed with (a)-(g) above.
 *
 *   Please note: This gives you special rights concerning the
 *   libraries which normally accompany the above pieces of software.
 *   It gives you no special rights concerning software that you write
 *   yourself.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than the libraries
 *   normally distributed with (a)-(g) above.
 *
 *   If you modify this file, you may extend this exception to your
 *   version of the file, but you are not obligated to do so. If you
 *   do not wish to do so, delete this exception statement from your
 *   version.
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 *
 *   Website
 *   =======
 *
 *   Emdros has a website here:
 *
 *   http://emdros.org
 *
 *
 *
 **************************************************************************/

#include <templatelang_classes.h>
#include "templatelangpars.h"
#include <memobject.h>

//////////////////////////////////////////////////////////////////
//
// TemplateASTNode
//
//////////////////////////////////////////////////////////////////


TemplateASTNode::TemplateASTNode()
{
	m_pNext = 0;
}


TemplateASTNode::~TemplateASTNode()
{
	delete m_pNext;
}


void TemplateASTNode::exec(TemplateLangExecEnv *pEE)
{
	if (m_pNext) {
		m_pNext->exec(pEE);
	}
}





//////////////////////////////////////////////////////////////////
//
// TemplateSimpleNode
//
//////////////////////////////////////////////////////////////////


TemplateSimpleNode::TemplateSimpleNode(eTemplateSimpleNodeKind kind)
{
	m_kind = kind;
}


TemplateSimpleNode::~TemplateSimpleNode()
{
	// Nothing to do
}


void TemplateSimpleNode::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	switch (m_kind) {
	case kTemplateID_D:
		pEE->addToOutput(id_d2string(pEE->m_pObject->getID_D()));
		break;
	case kTemplateFirstMonad:
		pEE->addToOutput(monad_m2string(pEE->m_pObject->first()));
		break;
	case kTemplateDBName:
		pEE->addToOutput(pEE->getDBName());
		break;
	}
}




//////////////////////////////////////////////////////////////////
//
// TemplateFeature
//
//////////////////////////////////////////////////////////////////

TemplateFeature::TemplateFeature(long index, bool bMangleFeature)
{
	m_index = index;
	m_bMangleFeature = bMangleFeature;
}


TemplateFeature::~TemplateFeature()
{
}


void TemplateFeature::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	if (m_bMangleFeature) {
		pEE->addToOutput(escapeXMLEntities(pEE->m_pObject->getFeature(m_index)));
	} else {
		pEE->addToOutput(pEE->m_pObject->getFeature(m_index));
	}
}




//////////////////////////////////////////////////////////////////
//
// TemplateAttrib
//
//////////////////////////////////////////////////////////////////

TemplateAttrib::TemplateAttrib(std::string *pString)
{
	m_attrib_name = *pString;
	delete pString;
}


TemplateAttrib::~TemplateAttrib()
{
}


void TemplateAttrib::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	pEE->addToOutput(escapeXMLEntities(pEE->getAttribute(m_attrib_name)));
}





//////////////////////////////////////////////////////////////////
//
// TemplateCounter
//
//////////////////////////////////////////////////////////////////

TemplateCounter::TemplateCounter(std::string *pString)
{
	m_counter_name = *pString;
	delete pString;
}


TemplateCounter::~TemplateCounter()
{
}


void TemplateCounter::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	pEE->addToOutput(long2string(pEE->getCounter(m_counter_name)));
}





//////////////////////////////////////////////////////////////////
//
// TemplateIncCounter
//
//////////////////////////////////////////////////////////////////

TemplateIncCounter::TemplateIncCounter(std::string *pString)
{
	m_counter_name = *pString;
	delete pString;
}


TemplateIncCounter::~TemplateIncCounter()
{
}


void TemplateIncCounter::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	pEE->incCounter(m_counter_name);
}





//////////////////////////////////////////////////////////////////
//
// TemplateString
//
//////////////////////////////////////////////////////////////////

TemplateString::TemplateString(std::string *pString)
{
	m_string = *pString;
	delete pString;
}


TemplateString::~TemplateString()
{
}


void TemplateString::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	pEE->addToOutput(m_string);
}




//////////////////////////////////////////////////////////////////
//
// TemplateIf
//
//////////////////////////////////////////////////////////////////


TemplateIf::TemplateIf(TemplateASTNode *pCondition,
		       TemplateASTNode *pExecIfTrue,
		       TemplateASTNode *pExecIfFalse)
{
	m_pCondition = pCondition;
	m_pExecIfTrue = pExecIfTrue;
	m_pExecIfFalse = pExecIfFalse;
}


TemplateIf::~TemplateIf()
{
	delete m_pCondition;
	delete m_pExecIfTrue;
	delete m_pExecIfFalse;
}


void TemplateIf::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	m_pCondition->exec(pEE);
	if (pEE->bLastIfCondition) {
		if (m_pExecIfTrue != 0) {
			m_pExecIfTrue->exec(pEE);
		}
	} else {	
		if (m_pExecIfFalse != 0) {
			m_pExecIfFalse->exec(pEE);
		}
	}
}




//////////////////////////////////////////////////////////////////
//
// TemplateIfFeatureEqual
//
//////////////////////////////////////////////////////////////////

TemplateIfFeatureEqual::TemplateIfFeatureEqual(long feature_index, std::string *pString)
{
	m_feature_index = feature_index;
	m_value = *pString;
	delete pString;
}


TemplateIfFeatureEqual::~TemplateIfFeatureEqual()
{
}


void TemplateIfFeatureEqual::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	bool bConditionResult = false;

	std::string featureValue = pEE->m_pObject->getFeature(m_feature_index);
	if (featureValue == m_value) {
		bConditionResult = true;
	} else {
		bConditionResult = false;
	}

	pEE->bLastIfCondition = bConditionResult;
}





//////////////////////////////////////////////////////////////////
//
// TemplateIfAttribEqual
//
//////////////////////////////////////////////////////////////////

TemplateIfAttribEqual::TemplateIfAttribEqual(std::string *pAttribName, std::string *pString)
{
	m_attrib_name = *pAttribName;
	delete pAttribName;
	m_value = *pString;
	delete pString;
}


TemplateIfAttribEqual::~TemplateIfAttribEqual()
{
}


void TemplateIfAttribEqual::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	bool bConditionResult = false;

	std::string attribValue = pEE->getAttribute(m_attrib_name);
	if (attribValue == m_value) {
		bConditionResult = true;
	} else {
		bConditionResult = false;
	}

	pEE->bLastIfCondition = bConditionResult;
}




//////////////////////////////////////////////////////////////////
//
// TemplateIfHasAttrib
//
//////////////////////////////////////////////////////////////////

TemplateIfHasAttrib::TemplateIfHasAttrib(std::string *pAttribName)
{
	m_attrib_name = *pAttribName;
	delete pAttribName;
}


TemplateIfHasAttrib::~TemplateIfHasAttrib()
{
}


void TemplateIfHasAttrib::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	bool bConditionResult = pEE->hasAttribute(m_attrib_name);

	pEE->bLastIfCondition = bConditionResult;
}




//////////////////////////////////////////////////////////////////
//
// TemplateListAppend
//
//////////////////////////////////////////////////////////////////

TemplateListAppend::TemplateListAppend(std::string *pStrListID, TemplateASTNode *pContents)
{
	m_strListID = *pStrListID;
	delete pStrListID;
	m_pContents = pContents;
}


TemplateListAppend::~TemplateListAppend()
{
	delete m_pContents;
}


void TemplateListAppend::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	pEE->enterListMode(m_strListID);
	m_pContents->exec(pEE);
	pEE->exitListMode();
}




//////////////////////////////////////////////////////////////////
//
// TemplateListEmit
//
//////////////////////////////////////////////////////////////////

TemplateListEmit::TemplateListEmit(std::string *pStrListID, std::string *pStrSeparator)
{
	m_strListID = *pStrListID;
	delete pStrListID;

	m_strSeparator = *pStrSeparator;
	delete pStrSeparator;
}


TemplateListEmit::~TemplateListEmit()
{
}


void TemplateListEmit::exec(TemplateLangExecEnv *pEE)
{
	TemplateASTNode::exec(pEE);

	pEE->emitList(m_strListID, m_strSeparator);
}






//////////////////////////////////////////////////////////////////
//
// TemplateLangExecEnv
//
//////////////////////////////////////////////////////////////////

TemplateLangExecEnv::TemplateLangExecEnv()
{
	m_pObject = 0;
	m_pError = new std::string();
	m_pNode = 0;
	m_pOutput = new std::string();
	pLexer = 0;
	clean();
}


TemplateLangExecEnv::~TemplateLangExecEnv()
{
	delete m_pError;
	delete m_pNode;
	delete m_pOutput;
	delete pLexer;
}

void TemplateLangExecEnv::clean()
{
	m_counters.clear();
	m_lists.clear();
	m_bIsAddingToList = false;
	m_strCurrentList = "";
	delete pLexer;
	pLexer = 0;
}


void TemplateLangExecEnv::setTopNode(TemplateASTNode *pNode)
{
	delete m_pNode;
	m_pNode = pNode;
}


TemplateASTNode *TemplateLangExecEnv::extractTopNode(void)
{
	TemplateASTNode *pResult = m_pNode;
	m_pNode = 0;
	return pResult;
}



void TemplateLangExecEnv::clearError(void)
{
	*m_pError = "";
}


std::string TemplateLangExecEnv::getError()
{
	return *m_pError;
}

void TemplateLangExecEnv::prependError(const std::string& addition)
{
	*m_pError = addition + "\n" + *m_pError;
}

void TemplateLangExecEnv::addToError(const std::string& addition)
{
	*m_pError += addition;
}



void TemplateLangExecEnv::clearOutput(void)
{
	*m_pOutput = "";
}


std::string TemplateLangExecEnv::getOutput()
{
	return *m_pOutput;
}


void TemplateLangExecEnv::addToOutput(const std::string& addition)
{
	if (m_bIsAddingToList) {
		m_lists[m_strCurrentList].push_back(addition);
	} else {
		*m_pOutput += addition;
	}
}



void TemplateLangExecEnv::clear()
{
	clearOutput();
	clearError();
}

void TemplateLangExecEnv::setAttributes(const AttributeMap& attrs)
{
	m_attrs = attrs;
}

std::string TemplateLangExecEnv::getAttribute(const std::string& attrib_name)
{
	if (m_attrs.find(attrib_name) != m_attrs.end()) {
		return m_attrs[attrib_name];
	} else {
		return "";
	}
}


bool TemplateLangExecEnv::hasAttribute(const std::string& attrib_name)
{
	if (m_attrs.find(attrib_name) != m_attrs.end()) {
		return true;
	} else {
		return false;
	}
}

long TemplateLangExecEnv::getCounter(const std::string& counter_name)
{
	CounterMap::const_iterator it = m_counters.find(counter_name);
	if (it == m_counters.end()) {
		m_counters[counter_name] = 0;

		return 0;
	} else {
		return it->second;
	}
}


void TemplateLangExecEnv::incCounter(const std::string& counter_name)
{
	CounterMap::iterator it = m_counters.find(counter_name);
	if (it == m_counters.end()) {
		m_counters[counter_name] = 1; // We start at 0, so after incrementing, 0 + 1 == 1.
	} else {
		++(it->second);
	}

}


void TemplateLangExecEnv::enterListMode(const std::string& list_name)
{
	m_bIsAddingToList = true;
	m_strCurrentList = list_name;
	if (m_lists.find(m_strCurrentList) == m_lists.end()) {
		m_lists.insert(std::make_pair(m_strCurrentList, std::list<std::string>()));
	}
}


void TemplateLangExecEnv::exitListMode()
{
	m_bIsAddingToList = false;
	m_strCurrentList = "";
}


void TemplateLangExecEnv::emitList(const std::string& list_name, const std::string& separator)
{
	StringListMap::const_iterator ci = m_lists.find(list_name);
	if (ci == m_lists.end()) {
		addToError("Error: Could not emit list '" + list_name + "' with separator '" + separator + "'");
	} else {
		// joinList is defined in <string_func.h>.
		*m_pOutput += joinList(separator, ci->second);
	}
}



/////////////////////////////////////////////////////////////////
//
// parser
//
/////////////////////////////////////////////////////////////////

Token *TemplateLangnewToken()
{
	Token *pToken = new Token;
	pToken->pString = 0;
	return pToken;
}

int ttlex(void *lvalp, void *parm)
{
	return ((TemplateLangExecEnv*) parm)->pLexer->scan((Token*) lvalp);
}


// Prototypes
extern void *TemplateLangParserAlloc(void *(*mallocProc)(size_t));
extern void TemplateLangParserFree(void *p, void (*freeProc)(void*));
extern void TemplateLangParser(
		      void *yyp,                       /* The parser */
		      int yymajor,                     /* The major token code number */
		      Token *yyminor,                  /* The value for the token */
		      TemplateLangExecEnv *pArg /* optional argument */
		      );



int ttparse(TemplateLangExecEnv *pEE)
{
	void *pParser;
	Token *pToken;
	int hTokenId;

	pEE->bSyntaxError = false;
	pParser = TemplateLangParserAlloc( malloc );
	pToken = TemplateLangnewToken();
	while((hTokenId = ttlex(pToken, pEE))
	      && !pEE->bSyntaxError) {
		TemplateLangParser(pParser, hTokenId, pToken, pEE);
		pToken = TemplateLangnewToken();
	}
	TemplateLangParser(pParser, 0, pToken, pEE);
	TemplateLangParserFree(pParser, free );
	delete pToken;

	if (pEE->bSyntaxError) {
		return 1;
	} else {
		return 0;
	}
}




TemplateASTNode *parseTemplate(TemplateLangExecEnv* pEE, const std::string& template_string, bool& bResult)
{
	pEE->clean();

	pEE->pLexer = new TemplateLanguageScanner(template_string);

	// Clear error
	pEE->clearError();

	// Declare return_value
	TemplateASTNode *pTemplateLangResult = 0;
  
	// Initialize bResult
	bResult = true;

	// Parser
	int iResult = ttparse(pEE);

	// Did parsing fail?
	if (iResult == 1) {
		bResult = false;
		pEE->prependError("Parsing failed\n");
	} else if (iResult == 2) {
		bResult = false;
		pEE->prependError("Parsing stack overflow. Parsing failed.\n");
	}

	// Was the parsing a success?
	if (bResult) {
		// If so, execute statements
		pTemplateLangResult = pEE->extractTopNode();
	} else {
		pTemplateLangResult = 0;
	}

	// Return
	return pTemplateLangResult;
}

